//  BMPx - The Dumb Music Player
//  Copyright (C) 2005 BMPx development team.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  --
//
//  The BMPx project hereby grants permission for non-GPL compatible GStreamer
//  plugins to be used and distributed together with GStreamer and BMPx. This
//  permission is above and beyond the permissions granted by the GPL license
//  BMPx is covered by.

#ifndef BMP_LIBRARY_UI_HPP 
#define BMP_LIBRARY_UI_HPP

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glibmm.h>
#include <glibmm/markup.h>
#include <gtkmm.h>
#include <libglademm.h>

#include <mcs/mcs.h>

#ifdef HAVE_HAL
#  include "hal.hpp"
#endif

#include "amazon.hpp"
#include "library.hpp"

#include "x_system_control.hpp"

namespace Bmp
{
  class TreeViewXDS;

  class LibraryUI
    : public Gtk::Window
  {
    public:

    LibraryUI (BaseObjectType                       *cobject,
               const Glib::RefPtr<Gnome::Glade::Xml> &xml);

    static LibraryUI*
    create ();

    ~LibraryUI ();

    virtual void
    present ()
    {
      Gtk::Window::present ();
      if (!updated)
      {
        b_import_tracks->set_sensitive (false);
        albums_update ();
        b_import_tracks->set_sensitive (true);
        updated = true;
      }
    }

    protected:

      virtual bool
      on_key_press_event (GdkEventKey*);

    private:

      typedef std::map < Glib::ustring, Gtk::TreeModel::iterator> IterMap;

      class Tracks
        : public Gtk::TreeModel::ColumnRecord
      {
        public:

          Gtk::TreeModelColumn<int>           track;
          Gtk::TreeModelColumn<Glib::ustring> title;
          Gtk::TreeModelColumn<Glib::ustring> artist;
          Gtk::TreeModelColumn<Glib::ustring> album;
          Gtk::TreeModelColumn<Glib::ustring> genre;
          Gtk::TreeModelColumn<int>           bitrate;
          Gtk::TreeModelColumn<int>           samplerate;
          Gtk::TreeModelColumn<Glib::ustring> uri;

#ifdef HAVE_HAL
          Gtk::TreeModelColumn<bool>          present;
#endif

          Tracks ()
          {
            add (track);
            add (title);
            add (artist);
            add (album);
            add (genre);
            add (bitrate);
            add (samplerate);
            add (uri);
#ifdef HAVE_HAL
            add (present);
#endif
          }
      };

      Tracks tracks;

      class Albums
        : public Gtk::TreeModel::ColumnRecord
      {
        public:
          Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > cover;
          Gtk::TreeModelColumn<Glib::ustring>          title;
          Gtk::TreeModelColumn<Glib::ustring>          artist;
          Gtk::TreeModelColumn<Glib::ustring>          album;
          Gtk::TreeModelColumn<bool>                   has_cover;
      
          Albums ()
          {
            add (cover);
            add (title);
            add (artist);
            add (album);
            add (has_cover);
          }
      };
      Albums albums; 

      class CBoxSearchCR
        : public Gtk::TreeModel::ColumnRecord
      {
        public:
          Gtk::TreeModelColumn<Glib::ustring> name;
          Gtk::TreeModelColumn<std::string> id;
      
          CBoxSearchCR ()
          {
            add (name);
            add (id);
          }
      };
      CBoxSearchCR cr_search; 
 
      typedef std::pair < Glib::ustring, Glib::ustring > RowKey; 
      typedef std::map < RowKey, Gtk::TreeModel::RowReference> RowMap;
      RowMap row_map;

#ifdef HAVE_HAL
      void
      hal_volume_removed  (Library::HALVolume volume);
      void
      hal_volume_added    (Library::HALVolume volume);
      void
      set_row_sensitive   (Gtk::CellRenderer* cell, const Gtk::TreeModel::iterator& m_iter);
      bool
      row_can_be_selected (const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool selected);
      bool
      all_tracks_present  ();
#endif

      enum LibraryPage
      {
        PAGE_ALBUMS,
        PAGE_SEARCH,
      };

      //---- Page Switch
      void
      update_track_view_from_albums ();
      void
      update_track_view_from_search ();
      void
      update_track_view (int id);

      //---- Playback
      void
      select_all    ();
      void
      enqueue       ();
      void
      play          ();
      void
      tracks_selection_changed ();


      //---- Removal/Adding/Modifications
      void
      import_tracks_into_album ();
      void
      import_tracks ();
      void
      import_tracks_as_album   ();

      void
      remove_tracks ();
      void
      remove_album  ();
      void
      fetch_cover   (const Gtk::TreePath &path);
      void
      move_album            ();
      void
      albums_update         ();
      void
      albums_modify         ();
      void
      tracks_modify         ();
      void
      albums_refilter       ();
      bool
      albums_row_is_visible     (const Gtk::TreeModel::const_iterator& iter);
      bool
      albums_event              (GdkEvent* event);
      int 
      albums_default_sort_func  (const Gtk::TreeModel::iterator& iter_a, const Gtk::TreeModel::iterator& iter_b);

      
      //---- Auxilliaries
      void
      notebook_page_switch (GtkNotebookPage *page, unsigned int page);
      void
      activate_default (const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column);
      void
      fill_tracks_from_rows (const Bmp::DB::VDataRow& vector);
      static gboolean
      shutdown_request (BmpSystemControl *control,
                        Bmp::LibraryUI   *ui)
      {
        return ui->busy;    
      }

      Glib::RefPtr<Gnome::Glade::Xml>     ref_xml;
      Glib::RefPtr<Gtk::ListStore>        store_tracks;
      Glib::RefPtr<Gtk::ListStore>        store_albums;
      Glib::RefPtr<Gtk::TreeModelFilter>  store_albums_filter;

      Bmp::TreeViewXDS        *view_albums;
      Gtk::TreeView           *view_tracks;

      Gtk::Entry              *albums_filter;
      Gtk::Notebook           *notebook_library;
      Gtk::Notebook           *notebook_add;

      Gtk::Button             *b_import_tracks_into_album;
      Gtk::Button             *b_import_tracks;
      Gtk::Button             *b_import_tracks_as_album;
      Gtk::Button             *b_remove_tracks;
      Gtk::Button             *b_remove_album;
      Gtk::Button             *b_albums_modify;
      Gtk::Button             *b_tracks_modify;
      Gtk::Button             *b_album_move;

      Gtk::Button             *b_close;
      Gtk::Button             *b_play;
      Gtk::Button             *b_enqueue;

      Gtk::ProgressBar        *progress_modify;

      Gtk::Label              *l_play;
      Gtk::Label              *l_enqueue;
      Gtk::Label              *l_current;
   
      enum FilterID
      {
        FILTER_1, 
        FILTER_2,
        FILTER_3,

        N_FILTERS,
      };

      // Search 
      Glib::RefPtr<Gtk::ListStore>     cs_store;
      Gtk::CheckButton                *cbutton[N_FILTERS];
      Gtk::ComboBox                   *cbox[N_FILTERS];
      Gtk::Entry                      *entry[N_FILTERS];

      void
      filter_toggled (FilterID id);

      // Shutdown watch stuff
      bool updated;
      bool busy;
      sigc::connection conn_notebook;

      // Misc
      Glib::RefPtr<Gdk::Pixbuf> disc;
      Glib::RefPtr<Gdk::Pixbuf> disc_small;
      Gtk::ComboBox *cb_view_size;
      Gtk::CellRendererText *cell_album_text;

      enum ViewSize
      {
        NORMAL,
        SMALL,
      };
      ViewSize viewsize; 

      void
      mcs_library_view_size_changed (MCS_CB_DEFAULT_SIGNATURE);
      bool init;
  };
}
#endif
