/*  BMPx - The Dumb Music Player
 *  Copyright (C) 2005 BMPx development team.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 *
 * The BMPx project hereby grant permission for non-gpl compatible GStreamer
 * plugins to be used and distributed together with GStreamer and BMPx. This
 * permission are above and beyond the permissions granted by the GPL license
 * BMPx is covered by.
 */

#ifndef SERVICE_CORE_HPP 
#define SERVICE_CORE_HPP

#include <glibmm.h>
#include <glib.h>
#include <boost/any.hpp>

#include "tracklist.hpp"
#include "flow.hpp"
#include "main.hpp"

namespace Bmp
{

    typedef sigc::slot <void> Slot_VOID;

#ifndef DOXYGEN_SHOULD_SKIP_THIS  
    struct CServiceCore;
#endif

    class ServiceCore
	: public Glib::Object
    {
      public:

	ServiceCore ();
	~ServiceCore ();

	//Application Control
	void
	register_application_control (const Slot_VOID& startup_,
				      const Slot_VOID& shutdown_)
	{
	  startup = startup_;
	  shutdown = shutdown_;
	}

	void
	run  () { startup (); }

	void
	stop () { shutdown (); }

	void
	init_dbus ();

	//Tracklists
        Bmp::Tracklist*	
	set_active_tracklist (const std::string& name)
	{
	  tracklist = tracklists.find(name)->second; 
	  return tracklist;
	}

	Bmp::Tracklist*
	get_active_tracklist ()
	{
	  return tracklist;
	}

     private:

	//Application Control
	Slot_VOID startup;
	Slot_VOID shutdown;

	//Flow plugins
 	typedef std::map<std::string, Bmp::Plugin::FlowBase*> FlowPlugins;
	bool load_plugins (const std::string& path);
	FlowPlugins plugins;
	Bmp::Plugin::FlowBase *plugin;

	//DBus C GObject
	GObject* c_service_core;
 	friend CServiceCore* c_service_core_new (Bmp::ServiceCore &control);

	//Tracklists
	typedef std::map<std::string, Bmp::Tracklist*> Tracklists;
	Tracklists tracklists;
	Bmp::Tracklist *tracklist;
    };

#ifndef DOXYGEN_SHOULD_SKIP_THIS  
    struct CServiceCore
    {
      GObject parent;
      Bmp::ServiceCore *control;
    };
#endif

};

#endif // SERVICE_CORE_HPP
